﻿using System;
using Android.App;
using Android.Views;
using Android.Widget;
using DeepSound.Helpers.Fonts;

namespace DeepSound.Helpers.Utils
{
   public class EmptyStateInflater
   {
        public Button EmptyStateButton;
        public TextView EmptyStateIcon;
        public TextView DescriptionText;
        public TextView TitleText;

        public enum Type
        {
            NoConnection,
            NoSearchResult,
            SomeThingWentWrong,
            NoUsers,
            NoNotifications,
            NoMessage,
            NoBlock,
            NoPlaylist,
            NoAlbums,
            NoFavorites,
            NoLiked,
            NoArtists,
            NoRecentlyPlayed,
            NoSound,
            NoComments,
            NoPurchases,
        }

        public void InflateLayout(View inflated, Type type)
        {
            try
            {

                EmptyStateIcon = (TextView) inflated.FindViewById(Resource.Id.emtyicon);
                TitleText = (TextView) inflated.FindViewById(Resource.Id.headText);
                DescriptionText = (TextView) inflated.FindViewById(Resource.Id.seconderyText);
                EmptyStateButton = (Button) inflated.FindViewById(Resource.Id.button);

                if (type == Type.NoConnection)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon,IonIconsFonts.IosThunderstormOutline);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoConnection_TitleText);
                    DescriptionText.Text =Application.Context.GetText(Resource.String.Lbl_NoConnection_DescriptionText);
                    EmptyStateButton.Text = Application.Context.GetText(Resource.String.Lbl_NoConnection_Button);
                }
                else if (type == Type.NoSearchResult)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Search);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoSearchResult_TitleText);
                    DescriptionText.Text =Application.Context.GetText(Resource.String.Lbl_NoSearchResult_DescriptionText);
                    EmptyStateButton.Text = Application.Context.GetText(Resource.String.Lbl_NoSearchResult_Button);
                }
                else if (type == Type.SomeThingWentWrong)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Close);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_SomThingWentWrong_TitleText);
                    DescriptionText.Text =Application.Context.GetText(Resource.String.Lbl_SomThingWentWrong_DescriptionText);
                    EmptyStateButton.Text = Application.Context.GetText(Resource.String.Lbl_SomThingWentWrong_Button);
                }
                else if (type == Type.NoUsers)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Person);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoMoreUser);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoNotifications)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon,IonIconsFonts.AndroidNotifications);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoNotification_TitleText);
                    DescriptionText.Text =Application.Context.GetText(Resource.String.Lbl_NoNotification_DescriptionText);
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoMessage)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Chatbox);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoMessage_TitleText);
                    DescriptionText.Text = Application.Context.GetText(Resource.String.Lbl_NoMessage_DescriptionText) + " " + AppSettings.ApplicationName;
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoBlock)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Person);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoBlockUsers);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoPlaylist)
                { 
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Person);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoPlaylist_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoAlbums)
                { 
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Person);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoAlbums_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoFavorites)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.MusicNote);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoFavorites_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoLiked)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.MusicNote);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoLiked_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoRecentlyPlayed)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.MusicNote);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoRecentlyPlayed_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoSound)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.MusicNote);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoDataUser_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoArtists)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Person);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoArtists_TitleText);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoComments)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.Chatbubble);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoComments_TitleText);
                    DescriptionText.Text = Application.Context.GetText(Resource.String.Lbl_NoComments_DescriptionText);
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }
                else if (type == Type.NoPurchases)
                {
                    FontUtils.SetTextViewIcon(FontsIconFrameWork.IonIcons, EmptyStateIcon, IonIconsFonts.MusicNote);
                    TitleText.Text = Application.Context.GetText(Resource.String.Lbl_NoPurchasesSongsFound);
                    DescriptionText.Text = " ";
                    EmptyStateButton.Visibility = ViewStates.Gone;
                }

            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }

        }
   }
}